--[[
	Author: Elsia
	InCombatIndicator, based on Nudge.
--]]

local Default = {
	background = false,
	ooc = false,
	width		= 135,
	height		= 35,
	font = 
	{ textSize	= 12,
      textfont = "Friz Quadrata TT",
	},
	pos	= {},
	locked = true
}

local Colors = {
	outofcombat = { r=0.06, g=0.98, b=0, a=1 },
	incombat = { r=0.94, g=0, b=0.14, a=1 }
}

InCombatIndicator = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0")
local L = AceLibrary("AceLocale-2.2"):new("InCombatIndicator")
local SML = AceLibrary:HasInstance("SharedMedia-1.0") and AceLibrary("SharedMedia-1.0") or nil

function InCombatIndicator:OnInitialize()

	local Options = {
		type = "group",
		args = {
	        lock = {
	            name = "Lock",
	            type = "toggle",
	            desc = "Lock/Unlock the button.",
	            get = function() return self.db.profile.locked end,
	            set = function( v ) self.db.profile.locked = v end,
	            map = {[false] = "Unlocked", [true] = "Locked"},
				order = 20,
	            guiNameIsMap = true,
            },
			width = {
				name = "Width", type = 'range', min = 10, max = 500, step = 1,
				desc = "Set the width of the button.",
				order = 40,
				get = function() return self.db.profile.width end,
				set = function( v )
					self.db.profile.width = v
					self:Layout()
				end
			},
			height = {
				name = "Height", type = 'range', min = 5, max = 50, step = 1,
				desc = "Set the height of the button.",
				order = 60,
				get = function() return self.db.profile.height end,
				set = function( v )
					self.db.profile.height = v
					self:Layout()
				end
			},
			font = {
				name = "Font", type = 'group',
				desc = "Set the font size of different elements.",
				args = {
					text = {
						name = "Font size", type = 'range', min = 6, max = 32, step = 1,
						desc = "Set the font size on the button.",
						order = 80,
						get = function() return self.db.profile.font.textSize end,
						set = function( v )
							self.db.profile.font.textSize = v
							self:Layout()
						end
					}
				}
			},
			background = {
				name = "Background", type = "toggle",
				desc = "Enable/Disable Background.",
				order = 120,
	            get = function() return self.db.profile.background end,
	            set = function( v ) self.db.profile.background = v end,
	            map = {[false] = "Text only", [true] = "Text with background"},
	            guiNameIsMap = true,
			},
			ooc = {
				name = "OOC", type = "toggle",
				desc = "Enable/Disable OOC display.",
				order = 140,
	            get = function() return self.db.profile.ooc end,
	            set = function( v ) self.db.profile.ooc = v end,
	            map = {[false] = "In Combat Only", [true] = "IC and OOC displayed"},
	            guiNameIsMap = true,
			}
		}
	}
	
	if SML then
		Options.args.font.args.textfont = {
			name = "Font face",
			desc = "Typeface to use for the text.",
			order = 100,
			type = "text",
			get = function() return self.db.profile.font.textfont end,
			set = function( v )
				self.db.profile.font.textfont = v
				self:Layout()
			end,
			validate = SML:List("font"),
			order = 210
		}
	end

	self:RegisterDB("InCombatIndicatorDB")
	self:RegisterDefaults('profile', Default)
	self:RegisterChatCommand( {"/incombatindicator"}, Options )
	self:RegisterChatCommand( {"/ici"}, Options )
	
end

function InCombatIndicator:OnEnable()

	self:CreateFrameWork()
	
	self:ScheduleRepeatingEvent("InCombatIndicator", self.OnUpdate, 0.1, self)
	self.master:Show()
end

function InCombatIndicator:SavePosition()

	local x, y = self.master:GetLeft(), self.master:GetTop()
	local s = self.master:GetEffectiveScale()
	
	self.db.profile.pos.x = x * s
	self.db.profile.pos.y = y * s

end

function InCombatIndicator:SetPosition()

	if self.db.profile.pos.x then
		local x = self.db.profile.pos.x
		local y = self.db.profile.pos.y

		local s = self.master:GetEffectiveScale()

		self.master:ClearAllPoints()
		self.master:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x / s, y / s)
	else
		self.master:ClearAllPoints()
		self.master:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	end

end

function InCombatIndicator:CreateFrameWork()	

	self.master = CreateFrame("Frame", "InCombatIndicatorFrame", UIParent)
	self.master:Hide()
	
	self.master:SetMovable(true)

	if not self.db.profile.locked then
		self.master:EnableMouse(true)
	else
		self.master:EnableMouse(false)
	end
		
	self.master:RegisterForDrag("LeftButton")
	self.master:SetScript("OnDragStart", function() if not self.db.profile.locked then self["master"]:StartMoving() end end)
	self.master:SetScript("OnDragStop", function() self["master"]:StopMovingOrSizing() self:SavePosition() end)
		
	self.master.Range = self.master:CreateFontString(nil, "ARTWORK")
	self.master.Range:SetFontObject(TextStatusBarText)
    
    self:Layout()
	
end

function InCombatIndicator:Layout()

	local textfont = SML and SML:Fetch("font", self.db.profile.font.textfont) or "Fonts\\FRIZQT__.ttf"
	
	self.master:SetWidth( self.db.profile.width )
	self.master:SetHeight( self.db.profile.height )

	if(self.db.profile.background) then
		self.master:SetBackdrop({
			bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 8,
			edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", 
			edgeSize = 16,
			insets = {left = 4, right = 4, top = 4, bottom = 4},
		})
		self.master:SetBackdropBorderColor(TOOLTIP_DEFAULT_COLOR.r, TOOLTIP_DEFAULT_COLOR.g, TOOLTIP_DEFAULT_COLOR.b)
		self.master:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
	end
	
	
	self.master.Range:SetJustifyH("CENTER")
    if(self.db.profile.background) then
		local gameFont, _, oflags = GameFontHighlightSmall:GetFont()
		self.master.Range:SetFont( gameFont, self.db.profile.font.textSize, oflags )
	else
		local gameFont, _, oflags = self.master.Range:GetFont()
		if SML then
		  gameFont = textfont
		end
		self.master.Range:SetFont( gameFont, self.db.profile.font.textSize, "OUTLINE") --[, oflags )--]
	end
	self.master.Range:ClearAllPoints()
	self.master.Range:SetPoint("CENTER", self.master, "CENTER",0,0)
	if(self.db.profile.background) then
		self.master.Range:SetTextColor( 1,1,1 )
	else
		self.master.Range:SetTextColor( TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b )
	end
	
	self:SetPosition()
	
end

function InCombatIndicator:OnUpdate()
	
	local text

	if not self.db.profile.locked then
		self.master:EnableMouse(true)
	else
		self.master:EnableMouse(false)
	end

	if (InCombatLockdown()) then
			text = L["In Combat"]
			InCombatIndicator.index = "incombat"
	else
			text = L["Out of Combat"]
			InCombatIndicator.index = "outofcombat"
	end

	if(self.db.profile.background) then
		InCombatIndicator.master:SetBackdropColor( 
			Colors[InCombatIndicator.index].r,
			Colors[InCombatIndicator.index].g,
			Colors[InCombatIndicator.index].b,
			Colors[InCombatIndicator.index].a
		)
			
		InCombatIndicator.master:SetBackdropBorderColor( 
			Colors[InCombatIndicator.index].r,
			Colors[InCombatIndicator.index].g,
			Colors[InCombatIndicator.index].b,
			Colors[InCombatIndicator.index].a
		)

		InCombatIndicator.master:SetBackdropBorderColor( 
			1,
			1,
			1,
			1
		)
	else
		InCombatIndicator.master:SetBackdropColor( 
			0,
			0,
			0,
			0
		)
			
		InCombatIndicator.master:SetBackdropBorderColor( 
			0,
			0,
			0,
			0
		)
	
		if (self.db.profile.ooc or InCombatIndicator.index == "incombat") then
			InCombatIndicator.master.Range:SetTextColor( 
				Colors[InCombatIndicator.index].r,
				Colors[InCombatIndicator.index].g,
				Colors[InCombatIndicator.index].b,
				Colors[InCombatIndicator.index].a
			)
		else
			InCombatIndicator.master.Range:SetTextColor( 
				0,
				0,
				0,
				0
			)
		end
	end
	
	InCombatIndicator.master.Range:SetText( text )

end